# Generated by roxygen2: do not edit by hand

export(backend)
export(backend_env)
export(backend_file)
export(backend_keyrings)
export(backend_macos)
export(backend_secret_service)
export(backend_wincred)
export(default_backend)
export(has_keyring_support)
export(key_delete)
export(key_get)
export(key_get_raw)
export(key_list)
export(key_list_raw)
export(key_set)
export(key_set_with_raw_value)
export(key_set_with_value)
export(keyring_create)
export(keyring_delete)
export(keyring_is_locked)
export(keyring_list)
export(keyring_lock)
export(keyring_unlock)
importFrom(R6,R6Class)
importFrom(utils,URLdecode)
importFrom(utils,head)
importFrom(utils,tail)
useDynLib(keyring, .registration = TRUE)
