% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/backend-wincred.R
\name{b_wincred_get}
\alias{b_wincred_get}
\title{Get a key from a Wincred keyring}
\usage{
b_wincred_get(self, private, service, username, keyring)
}
\arguments{
\item{service}{Service name. Must not be empty.}

\item{username}{Username. Might be empty.
\enumerate{
\item We check if the key is on the default keyring.
\item If yes, we just return it.
\item Otherwise check if the keyring is locked.
\item If locked, then unlock it.
\item Get the AES key from the keyring.
\item Decrypt the key with the AES key.
}

Additionally, users may specify an encoding to use when converting the
password from a byte-string, for compatibility with other software such as
python's keyring package. This is done via an option, or an environment variable.}
}
\description{
Get a key from a Wincred keyring
}
\keyword{internal}
